<template>
	<NodeBasicLayout v-model:title="data.title" :type="NodeType.Code" :showOffset="false" :description="data.description">
		<Handle :id="targetHandleId" type="target" :position="Position.Left" />
		<FieldLayout :title="codeInput.name">
			<div class="flex flex-col gap-y-2">
				<div :key="subIndex" v-for="(subItem, subIndex) in codeInput.params[0].value" class="flex-items-center gap-x-2 pr-8">
					<el-input filterable class="w-[120px] flex-0" v-model="subItem.key" placeholder="参数名"> </el-input>

					<el-input filterable class="w-[120px] flex-0" v-model="subItem.value" placeholder="值"> </el-input>
					<span class="ywifont ywicon-shanchu text-red-400 cursor-pointer" @click="delInputVarItem(subIndex)"></span>
				</div>
				<el-button class="w-fit mt-3" type="primary" @click="addInputVarItem">添加新的入参</el-button>
			</div>
		</FieldLayout>
		<FieldLayout :title="codeStr.name">
			<div class="relative">
				<codemirror
					@scroll="handleCodeScroll"
					class="nowheel overflow-auto [&>.cm-editor]:min-h-[140px] [&>.cm-editor]:max-h-[340px] cursor-text"
					v-model="codeStr.params[0].value"
					:autofocus="true"
					:indent-with-tab="true"
					:tab-size="2"
					:extensions="javascriptEditorExtensions"
				/>
				<!-- <el-tooltip effect="dark" content="全屏" placement="top">
				
				</el-tooltip> -->
				<YWIcon
					class="absolute right-1 top-1 !text-gray-300 cursor-pointer"
					name="pingmufangda"
					@click="fullEditCodeClick(codeStr.params[0])"
				/>
			</div>
		</FieldLayout>
		<FieldLayout :title="codeOutput.name">
			<div class="flex flex-col gap-y-2">
				<div :key="subIndex" v-for="(subItem, subIndex) in codeOutput.params[0].value" class="flex-items-center gap-x-2 pr-8">
					<el-input filterable class="w-[120px] flex-0" v-model="subItem.key" placeholder="参数名"> </el-input>

					<el-select v-model="subItem.type">
						<el-option
							v-for="item in Object.keys(parameterTypeMap)"
							:key="item"
							:value="item"
							:label="parameterTypeMap[item]"
						></el-option>
					</el-select>
					<span class="ywifont ywicon-shanchu text-red-400 cursor-pointer" @click="delOutputVarItem(subIndex)"></span>
				</div>
				<el-button class="w-fit mt-3" type="primary" @click="addOutputVarItem">添加新的出参</el-button>
			</div>
		</FieldLayout>
		<Handle :id="sourceHandleId" type="source" :position="Position.Right" />
		<Teleport to="body">
			<CodeEditDialog v-if="editMapCodeItem" v-model="codeEditDlgIsShow" :item="editMapCodeItem"></CodeEditDialog>
		</Teleport>
	</NodeBasicLayout>
</template>

<script lang="ts" setup>
import type { NodeProps } from '@vue-flow/core';
import { Handle, Position, useNode } from '@vue-flow/core';
import { ref } from 'vue';
import { VueFlowHelper } from '../../VueFlowHelper';
import { NodeType, ParameterType, parameterTypeMap } from '../../vueFlowEnum';
import { javascript } from '@codemirror/lang-javascript';
import { vscodeDark } from '@uiw/codemirror-theme-vscode';
import { Codemirror } from 'vue-codemirror';
import CodeEditDialog from './components/CodeEditDlg.vue';
import FieldLayout from './components/FieldLayout.vue';
import NodeBasicLayout from './components/NodeBasicLayout.vue';
import { LLMNodeData, LLMNodeEvents } from './index';
defineProps<NodeProps<LLMNodeData, LLMNodeEvents>>();
const node = useNode();
const targetHandleId = ref(VueFlowHelper.getHandleId(node.node, 'target'));
const sourceHandleId = ref(VueFlowHelper.getHandleId(node.node, 'source'));

const javascriptEditorExtensions = [javascript(), vscodeDark];

const data = ref(node.node.data);
const codeInput = ref(VueFlowHelper.getGroupParam(data.value, 0));
const codeStr = ref(VueFlowHelper.getGroupParam(data.value, 1));
const codeOutput = ref(VueFlowHelper.getGroupParam(data.value, 2));
const handleCodeScroll = (e) => {
	e.stopPropagation();
};
const getInputEmptyItem = () => {
	return {
		key: '',
		type: 'input',
		label: '',
		value: '',
	};
};

const getOutputEmptyItem = () => {
	return {
		key: '',
		type: ParameterType.String,
	};
};
const delInputVarItem = (index) => {
	if (!codeInput.value.params[0].value || codeInput.value.params[0].value.length === 0) {
		return;
	}
	// if (codeInput.value.params[0].value?.length === 1) {
	// 	codeInput.value.params[0].value = getEmptyItem();
	// 	return;
	// }
	codeInput.value.params[0].value.splice(index, 1);
};

const addInputVarItem = () => {
	if (!codeInput.value.params[0].value || codeInput.value.params[0].value.length === 0) {
		codeInput.value.params[0].value = [];
	}
	codeInput.value.params[0].value.push(getInputEmptyItem());
};

const delOutputVarItem = (index) => {
	if (!codeOutput.value.params[0].value || codeOutput.value.params[0].value.length === 0) {
		return;
	}
	// if (codeOutput.value.params[0].value?.length === 1) {
	// 	codeOutput.value.params[0].value = getOutputEmptyItem();
	// 	return;
	// }
	codeOutput.value.params[0].value.splice(index, 1);
};

const addOutputVarItem = () => {
	if (!codeOutput.value.params[0].value || codeOutput.value.params[0].value.length === 0) {
		codeOutput.value.params[0].value = [];
	}
	codeOutput.value.params[0].value.push(getOutputEmptyItem());
};

const editMapCodeItem = ref({});
const codeEditDlgIsShow = ref(false);
const fullEditCodeClick = (item) => {
	editMapCodeItem.value = item;
	codeEditDlgIsShow.value = true;
};
</script>
